package com.smoa.comp.sdk;

import java.util.ArrayList;
import java.util.List;

import javax.xml.bind.JAXBException;
import javax.xml.namespace.QName;
import javax.xml.ws.Service;
import javax.xml.ws.WebServiceFeature;
import javax.xml.ws.handler.Handler;
import javax.xml.ws.wsaddressing.W3CEndpointReference;

import com.smoa.comp.sdk.ardl.ARDL;
import com.smoa.comp.sdk.exceptions.InvalidRequestMessageException;
import com.smoa.comp.sdk.exceptions.NoMatchingResourcesException;
import com.smoa.comp.sdk.exceptions.NoSlotsAvailableException;
import com.smoa.comp.sdk.exceptions.NotAuthorizedException;
import com.smoa.comp.sdk.exceptions.UnknownReservationIdentifierException;
import com.smoa.comp.sdk.exceptions.UnsupportedFeatureException;
import com.smoa.comp.sdk.types.ReservationDocument;
import com.smoa.comp.sdk.types.ReservationEndpointReference;
import com.smoa.comp.stubs.ares.ARESReservationDocumentType;
import com.smoa.comp.stubs.ares.ARESReservationStatusType;
import com.smoa.comp.stubs.ares.CreateReservationType;
import com.smoa.comp.stubs.ares.GetActiveReservationsType;
import com.smoa.comp.stubs.ares.GetReservationDocumentResponseType;
import com.smoa.comp.stubs.ares.GetReservationDocumentType;
import com.smoa.comp.stubs.ares.GetReservationStatusType;
import com.smoa.comp.stubs.ares.InvalidRequestMessageFault;
import com.smoa.comp.stubs.ares.NoMatchingResourcesFault;
import com.smoa.comp.stubs.ares.NoSlotsAvailableFault;
import com.smoa.comp.stubs.ares.NotAuthorizedFault;
import com.smoa.comp.stubs.ares.SMOAComputingAdvanceReservation;
import com.smoa.comp.stubs.ares.SMOAComputingAdvanceReservationPortType;
import com.smoa.comp.stubs.ares.TerminateReservationType;
import com.smoa.comp.stubs.ares.UnknownReservationIdentifierFault;
import com.smoa.comp.stubs.ares.UnsupportedFeatureFault;
import com.smoa.core.sdk.SMOACoreService;
import com.smoa.core.sdk.security.AuthenticationModule;


public class ARES extends SMOACoreService {

	public ARES(String hostname, int port, AuthenticationModule transportLayer,
			AuthenticationModule messageLayer, boolean useWSA)
			throws JAXBException {
		super(hostname, port, transportLayer, messageLayer, useWSA);
	}

	public ARES(String hostname, int port, AuthenticationModule transportLayer,
			AuthenticationModule messageLayer) throws JAXBException {
		super(hostname, port, transportLayer, messageLayer);
	}

	public ARES(String hostname, int port, AuthenticationModule am)
			throws JAXBException {
		super(hostname, port, am);
	}

	public static final String SMOA_ARES_NS = "http://schemas.smoa-project.com/comp/2009/01/ares";
	protected SMOAComputingAdvanceReservationPortType ares;
	
	public ARES(String hostname, int port, AuthenticationModule am,
			boolean useWSA) throws JAXBException{
		super(hostname, port, am, useWSA);
		ares = (SMOAComputingAdvanceReservationPortType)stubs;
	}

	@Override
	@SuppressWarnings("all")
	protected List<Handler> getHandlersChain() {
		return null;
	}

	@Override
	protected Object getPort(Service service, WebServiceFeature... features) {
		return ((SMOAComputingAdvanceReservation)service).getSMOAComputingAdvanceReservation(features);
	}

	@Override
	protected Service getService() {
		return new SMOAComputingAdvanceReservation(ARES.class.getResource("/schemas/smoa-comp-ares.wsdl"), new QName(SMOA_ARES_NS, "SMOAComputingAdvanceReservation"));
	}
	
	public ReservationEndpointReference createReservation(ARDL reservationDocument) throws InvalidRequestMessageException, NoMatchingResourcesException, NoSlotsAvailableException, NotAuthorizedException, UnsupportedFeatureException {
		CreateReservationType parameters = new CreateReservationType();
		
		parameters.setReservationDocument( new ARESReservationDocumentType() );
		parameters.getReservationDocument().setReservationDefinition(reservationDocument.getReservationDefinition());
		
		try {
			return new ReservationEndpointReference(ares.createReservation(parameters).getReservationIdentifier());
		} catch (InvalidRequestMessageFault e) {
			throw new InvalidRequestMessageException(e);
		} catch (NoMatchingResourcesFault e) {
			throw new NoMatchingResourcesException(e);
		} catch (NoSlotsAvailableFault e) {
			throw new NoSlotsAvailableException(e);
		} catch (NotAuthorizedFault e) {
			throw new NotAuthorizedException(e);
		} catch (UnsupportedFeatureFault e) {
			throw new UnsupportedFeatureException(e);
		}
	}
	
	public void terminateReservation(ReservationEndpointReference rEpr) throws NotAuthorizedException, UnknownReservationIdentifierException {
		TerminateReservationType parameters = new TerminateReservationType();
		
		parameters.setReservationIdentifier(rEpr.getEndpointReference(getURL()));
		
		try {
			ares.terminateReservation(parameters);
		} catch (NotAuthorizedFault e) {
			throw new NotAuthorizedException(e);
		} catch (UnknownReservationIdentifierFault e) {
			throw new UnknownReservationIdentifierException(e);
		}
	}
	
	public ARESReservationStatusType getReservationStatus(ReservationEndpointReference rEpr) throws NotAuthorizedException, UnknownReservationIdentifierException {
		GetReservationStatusType parameters = new GetReservationStatusType();
		
		parameters.setReservationIdentifier(rEpr.getEndpointReference(getURL()));
		
		try {
			return ares.getReservationStatus(parameters).getReservationStatus();
		} catch (NotAuthorizedFault e) {
			throw new NotAuthorizedException(e);
		} catch (UnknownReservationIdentifierFault e) {
			throw new UnknownReservationIdentifierException(e);
		}
	}
	
	public ReservationDocument getReservationDocument(ReservationEndpointReference rEpr) throws NotAuthorizedException, UnknownReservationIdentifierException {
		GetReservationDocumentType parameters = new GetReservationDocumentType();
		
		parameters.setReservationIdentifier(rEpr.getEndpointReference(getURL()));
		GetReservationDocumentResponseType rdt;
		try {
			rdt = ares.getReservationDocument(parameters);
			return new ReservationDocument(new ARDL(rdt.getReservationDocument().getReservationDefinition()), rdt.getLocalReservationId());
		} catch (NotAuthorizedFault e) {
			throw new NotAuthorizedException(e);
		} catch (UnknownReservationIdentifierFault e) {
			throw new UnknownReservationIdentifierException(e);
		}
	}
	
	public List<ReservationEndpointReference> getActiveReservations() throws NotAuthorizedException {
		
		List<W3CEndpointReference> w3cList;
		try {
			w3cList = ares.getActiveReservations(new GetActiveReservationsType()).getReservationIdentifier();
			List<ReservationEndpointReference> fedList = new ArrayList<ReservationEndpointReference>(w3cList.size());
			
			for (W3CEndpointReference epr : w3cList) {
				fedList.add(new ReservationEndpointReference(epr));
			}
			
			return fedList;
		} catch (NotAuthorizedFault e) {
			throw new NotAuthorizedException(e);
		}

	}

}
