package com.smoa.comp.sdk;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.security.Security;
import java.util.ArrayList;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.regex.Pattern;

import javax.activation.DataHandler;
import javax.activation.DataSource;
import javax.xml.bind.JAXBException;
import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.datatype.DatatypeConstants;
import javax.xml.datatype.DatatypeFactory;
import javax.xml.datatype.XMLGregorianCalendar;
import javax.xml.namespace.QName;
import javax.xml.ws.BindingProvider;
import javax.xml.ws.Service;
import javax.xml.ws.WebServiceFeature;
import javax.xml.ws.handler.Handler;
import javax.xml.ws.handler.MessageContext;

import org.metastatic.rsync.Checksum32;
import org.metastatic.rsync.ChecksumPair;
import org.metastatic.rsync.Configuration;
import org.metastatic.rsync.GeneratorEvent;
import org.metastatic.rsync.GeneratorListener;
import org.metastatic.rsync.GeneratorStream;
import org.metastatic.rsync.JarsyncProvider;
import org.metastatic.rsync.ListenerException;
import org.metastatic.rsync.Rdiff;

import com.smoa.comp.sdk.jsdl.JSDL;
import com.smoa.comp.sdk.rsync.DeltaDataSource;
import com.smoa.comp.sdk.rsync.DeltaInputStream;
import com.smoa.comp.sdk.rsync.GetDeltaHandler;
import com.smoa.comp.sdk.rsync.Params;
import com.smoa.comp.stubs.rsync.BlockSignatureType;
import com.smoa.comp.stubs.rsync.CommonType;
import com.smoa.comp.stubs.rsync.FileType;
import com.smoa.comp.stubs.rsync.FilesType;
import com.smoa.comp.stubs.rsync.IOFault;
import com.smoa.comp.stubs.rsync.InternalFault;
import com.smoa.comp.stubs.rsync.InvalidPathFault;
import com.smoa.comp.stubs.rsync.RsyncFault;
import com.smoa.comp.stubs.rsync.SMOAComputingRsync;
import com.smoa.comp.stubs.rsync.SMOAComputingRsyncPortType;
import com.smoa.comp.stubs.rsync.SignatureType;
import com.smoa.core.sdk.SMOACoreService;
import com.smoa.core.sdk.attachments.AttachmentHandler;
import com.smoa.core.sdk.security.AuthenticationModule;

public class SMOARsync extends SMOACoreService {

    public SMOARsync(String hostname, int port,
			AuthenticationModule transportLayer,
			AuthenticationModule messageLayer, boolean useWSA)
			throws JAXBException {
		super(hostname, port, transportLayer, messageLayer, useWSA);
	}

	public SMOARsync(String hostname, int port,
			AuthenticationModule transportLayer,
			AuthenticationModule messageLayer) throws JAXBException {
		super(hostname, port, transportLayer, messageLayer);
	}

	public SMOARsync(String hostname, int port, AuthenticationModule am)
			throws JAXBException {
		super(hostname, port, am);
	}

	public static final String SMOA_RSYNC_NS = "http://schemas.smoa-project.com/comp/2010/04/rsync";
    

    protected static final sun.misc.BASE64Encoder BASE64_ENCODER = new sun.misc.BASE64Encoder();
    protected static final sun.misc.BASE64Decoder BASE64_DECODER = new sun.misc.BASE64Decoder();

    protected SMOAComputingRsyncPortType rsync;
    protected AttachmentHandler attachmentHandler;

    protected Rdiff rdiff = new Rdiff();

    private class LazyInputStream extends InputStream {

	private FileInputStream file = null;

	public LazyInputStream(String filename) {
	    try {
		file = new FileInputStream(filename);
	    } catch (FileNotFoundException e) {
	    }
	}

	@Override
	public int read() throws IOException {
	    return file != null ? file.read() : -1;
	}

    }

    public SMOARsync(String hostname, int port, AuthenticationModule am,
	    boolean useWSA) throws JAXBException {
	super(hostname, port, am, useWSA);
	this.rsync = (SMOAComputingRsyncPortType) stubs;
    }

    @Override
    protected Object getPort(Service service, WebServiceFeature... features) {
	return ((SMOAComputingRsync) service).getSMOAComputingRsync(features);
    }

    @Override
    protected Service getService() {
	return new SMOAComputingRsync(
		SMOARsync.class.getResource("/schemas/smoa-comp-rsync.wsdl"),
		new QName(SMOA_RSYNC_NS, "SMOAComputingRsync"));
    }

    private IOException stagingException;

    @Override
    @SuppressWarnings("all")
    protected List<Handler> getHandlersChain() {
	attachmentHandler = new AttachmentHandler(
		new AttachmentHandler.IOExceptionListener() {

		    public void exceptionThrown(IOException ex) {
			stagingException = ex;
		    }
		});

	List<Handler> handlers = new ArrayList<Handler>();
	handlers.add(attachmentHandler);

	return handlers;
    }

    protected void checkForIOException() throws IOException {
	if (stagingException != null) {
	    IOException ioEx = stagingException;
	    stagingException = null;
	    throw ioEx;
	}
    }

    protected String joinPath(String a, String b) {
	// The File.pathSeparator is not used to ensure compatibility with
	// smoa-comp-rsync, which has / hardcoded
	if (!a.endsWith("/"))
	    a += "/";
	return a + b;
    }

    private FilesType getLocalFiles(String basedir, String path)
	    throws DatatypeConfigurationException {
	FilesType localFiles = new FilesType();
	File dir = new File(joinPath(basedir, path));
	for (String s : dir.list()) {
	    s = joinPath(path, s);
	    File f = new File(joinPath(basedir, s));
	    if (f.isDirectory())
		localFiles.getFile()
			.addAll(getLocalFiles(basedir, s).getFile());
	    else {
		FileType file = new FileType();
		file.setPath(s);
		file.setMTime(timestampToXML(f.lastModified()));
		localFiles.getFile().add(file);
	    }
	}
	return localFiles;
    }

    private XMLGregorianCalendar timestampToXML(long time)
	    throws DatatypeConfigurationException {
	DatatypeFactory factory = DatatypeFactory.newInstance();
	GregorianCalendar cal = new GregorianCalendar();
	cal.setTime(new Date(time));
	XMLGregorianCalendar xml = factory.newXMLGregorianCalendar(cal);
	return xml;
    }

    private class SignatureGeneratorListener implements GeneratorListener {

	private SignatureType result;

	public SignatureGeneratorListener(SignatureType result) {
	    this.result = result;
	}

	public void update(GeneratorEvent event) throws ListenerException {
	    ChecksumPair pair = event.getChecksumPair();
	    BlockSignatureType blockSig = new BlockSignatureType();
	    blockSig.setWeak(pair.getWeak());
	    blockSig.setStrong(BASE64_ENCODER.encode(pair.getStrong()));
	    result.getSignatures().add(blockSig);
	}

    }

    private SignatureType GetSignature(String fullpath) throws IOException,
	    NoSuchAlgorithmException {
	Configuration c = new Configuration();
	c.strongSum = MessageDigest.getInstance("MD4");
	c.weakSum = new Checksum32(Params.CHAR_OFFSET);
	c.blockLength = Params.BLOCK_LENGTH;
	c.strongSumLength = Params.STRONG_SUM_LENGTH;
	SignatureType result = new SignatureType();
	result.setBlockLength(c.blockLength);
	File f = new File(fullpath);
	result.setLength((int) f.length());
	result.setRemainder((int) f.length() % c.blockLength);
	result.setStrongSumLength(c.strongSumLength);
	GeneratorStream gen = new GeneratorStream(c);
	gen.addListener(new SignatureGeneratorListener(result));
	int len = 0;
	InputStream in = new LazyInputStream(fullpath);
	byte[] buf = new byte[Params.CHUNK_SIZE];
	while ((len = in.read(buf)) != -1) {
	    try {
		gen.update(buf, 0, len);
	    } catch (ListenerException le) {
		throw (IOException) le.getCause();
	    }
	}
	try {
	    gen.doFinal();
	} catch (ListenerException le) {
	    throw (IOException) le.getCause();
	}
	return result;
    }
  
    private void putDelta(CommonType common, String localBasedir, String path)
	    throws IOFault, InternalFault, RsyncFault, IOException,
	    NoSuchAlgorithmException {
	System.out.println("Sending file: " + path);
	List<ChecksumPair> signatures = getSignatures(common, path);
	Map<String, DataHandler> attachments = new HashMap<String, DataHandler>();	
	DataSource delta = new DeltaDataSource(new DeltaInputStream(signatures, new LazyInputStream(joinPath(localBasedir, path))));
	attachments.put("delta", new DataHandler(delta));
	((BindingProvider) rsync).getRequestContext().put(
		MessageContext.OUTBOUND_MESSAGE_ATTACHMENTS, attachments);
	rsync.putDelta(common, path);
	((BindingProvider) rsync).getRequestContext().put(
		MessageContext.OUTBOUND_MESSAGE_ATTACHMENTS, null);
	checkForIOException();
    }

    private List<ChecksumPair> getSignatures(CommonType common, String path)
	    throws IOFault, InternalFault, RsyncFault, IOException {
	SignatureType sig = rsync.getSignature(common, path);
	List<ChecksumPair> signatures = new ArrayList<ChecksumPair>();
	long offset = 0;
	for (BlockSignatureType block : sig.getSignatures()) {
	    signatures.add(new ChecksumPair(block.getWeak(), BASE64_DECODER
		    .decodeBuffer(block.getStrong()), offset));
	    offset += sig.getBlockLength();
	}
	return signatures;
    }

    private void getDelta(CommonType common, String localBasedir, String path)
	    throws NoSuchAlgorithmException, IOException {
	System.out.println("Receiving file: " + path);
	String fullpath = new File(joinPath(localBasedir, path))
		.getAbsolutePath();
	SignatureType sig = GetSignature(fullpath);
	this.attachmentHandler.setFileStagingHandler(new GetDeltaHandler(
		fullpath));
	rsync.getDelta(common, path, sig);
	this.attachmentHandler.setFileStagingHandler(null);
	checkForIOException();
    }
    
    private boolean fileOk(String path, List<Pattern> exclude) {

	if (exclude != null) {

	    if (path.startsWith("/"))
		path = path.substring(1);

	    for (Pattern p : exclude) {

		if (p.matcher(path).matches())
		    return false;

	    }

	}

	return true;
    }

    public void synchronize(JSDL jsdl, String localDirectory, List<Pattern> exclude)
	    throws InternalFault, InvalidPathFault,
	    DatatypeConfigurationException, NoSuchAlgorithmException,
	    IOException, IOFault, RsyncFault {
	Security.addProvider(new JarsyncProvider());
	CommonType params = new CommonType();
	params.setBaseDir(jsdl.getWorkingDirectory());
	params.setUserName(jsdl.getUserName());
	FilesType remoteFiles = rsync.getFiles(params);
	FilesType localFiles = getLocalFiles(localDirectory, "");
	for (FileType local : localFiles.getFile()) {
	    boolean found = false;	    
	    
	    if(!fileOk(local.getPath(), exclude))
		continue;
	    	    
	    for (FileType remote : remoteFiles.getFile()) {
		if (local.getPath().equals(remote.getPath())) {
		    found = true;
		    if (local.getMTime().compare(remote.getMTime()) == DatatypeConstants.LESSER)
			getDelta(params, localDirectory, local.getPath());
		    else
			putDelta(params, localDirectory, local.getPath());
		}
	    }
	    if (!found)
		putDelta(params, localDirectory, local.getPath());
	}
	for (FileType remote : remoteFiles.getFile()) {
	    boolean found = false;
	    
	    if(!fileOk(remote.getPath(), exclude))
		continue;
	    
	    for (FileType local : localFiles.getFile()) {
		if (local.getPath().equals(remote.getPath())) {
		    found = true;
		    break;
		}
	    }
	    if (!found)
		getDelta(params, localDirectory, remote.getPath());
	}
    }

    public void localToRemote(JSDL jsdl, String localDirectory, List<Pattern> exclude)
	    throws InternalFault, InvalidPathFault,
	    DatatypeConfigurationException, NoSuchAlgorithmException,
	    IOException, IOFault, RsyncFault {
	Security.addProvider(new JarsyncProvider());
	CommonType params = new CommonType();
	params.setBaseDir(jsdl.getWorkingDirectory());
	params.setUserName(jsdl.getUserName());
	FilesType localFiles = getLocalFiles(localDirectory, "");
	for (FileType local : localFiles.getFile()) {
	    
	    if(fileOk(local.getPath(), exclude))
		putDelta(params, localDirectory, local.getPath());
	    
	}
    }

    public void remoteToLocal(JSDL jsdl, String localDirectory, List<Pattern> exclude)
	    throws InternalFault, InvalidPathFault,
	    DatatypeConfigurationException, NoSuchAlgorithmException,
	    IOException, IOFault, RsyncFault {
	Security.addProvider(new JarsyncProvider());
	CommonType params = new CommonType();
	params.setBaseDir(jsdl.getWorkingDirectory());
	params.setUserName(jsdl.getUserName());
	FilesType remoteFiles = rsync.getFiles(params);
	
	for (FileType remote : remoteFiles.getFile()) {
	    	    
	    if(fileOk(remote.getPath(), exclude))
		getDelta(params, localDirectory, remote.getPath());
	    
	}
    }

}
