package com.smoa.comp.sdk.rsync;

import java.io.DataInputStream;
import java.io.EOFException;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.io.RandomAccessFile;

import org.metastatic.rsync.DataBlock;
import org.metastatic.rsync.Delta;
import org.metastatic.rsync.Offsets;
import org.metastatic.rsync.Rdiff;

import com.smoa.core.sdk.attachments.FileStagingHandler;

public class GetDeltaHandler implements FileStagingHandler {

    private String origin;
    private RandomAccessFile in = null;
    private RandomAccessFile out = null;

    public GetDeltaHandler(String origin) throws FileNotFoundException,
	    IOException {
	this.origin = origin;
    }

    public InputStream stageIn(String sourceFileName) throws IOException {
	return null;
    }

    protected void applyDelta(Delta o) throws IOException {
	if (o instanceof DataBlock) {
	    long off = ((DataBlock) o).getOffset();
	    out.seek(off);
	    out.write(((DataBlock) o).getData());
	} else if (o instanceof Offsets) {
	    if (in == null) {
		throw new IOException(
			"original file does not exist or not readable");
	    }
	    int len = ((Offsets) o).getBlockLength();
	    long off1 = ((Offsets) o).getOldOffset();
	    long off2 = ((Offsets) o).getNewOffset();
	    byte[] buf = new byte[len];
	    in.seek(off1);
	    in.read(buf);
	    out.seek(off2);
	    out.write(buf);
	}
    }

    private static long readInt(int len, InputStream in) throws IOException {
	long i = 0;
	for (int j = len - 1; j >= 0; j--) {
	    int k = in.read();
	    if (k == -1)
		throw new EOFException();
	    i |= (k & 0xff) << 8 * j;
	}
	return i;
    }

    private static int readInt(InputStream in) throws IOException {
	int i = 0;
	for (int j = 3; j >= 0; j--) {
	    int k = in.read();
	    if (k == -1)
		throw new EOFException();
	    i |= (k & 0xff) << 8 * j;
	}
	return i;
    }

    long offset = 0;
    DataInputStream data;

    protected Delta readDelta() throws IOException {

	int command;
	byte[] buf;
	if ((command = data.read()) != -1) {
	    switch (command) {
	    case Rdiff.OP_END:
		return null;

	    case Rdiff.OP_LITERAL_N1: {
		buf = new byte[(int) readInt(1, data)];
		data.readFully(buf);
		DataBlock db = new DataBlock(offset, buf);
		offset += buf.length;
		return db;
	    }

	    case Rdiff.OP_LITERAL_N2: {
		buf = new byte[(int) readInt(2, data)];
		data.readFully(buf);
		DataBlock db = new DataBlock(offset, buf);
		offset += buf.length;
		return db;
	    }

	    case Rdiff.OP_LITERAL_N4: {
		buf = new byte[(int) readInt(4, data)];
		data.readFully(buf);
		DataBlock db = new DataBlock(offset, buf);
		offset += buf.length;
		return db;
	    }

	    case Rdiff.OP_COPY_N1_N1: {
		int oldOff = (int) readInt(1, data);
		int bs = (int) readInt(1, data);
		Offsets o = new Offsets(oldOff, offset, bs);
		offset += bs;
		return o;
	    }

	    case Rdiff.OP_COPY_N1_N2: {
		int oldOff = (int) readInt(1, data);
		int bs = (int) readInt(2, data);
		Offsets o = new Offsets(oldOff, offset, bs);
		offset += bs;
		return o;
	    }

	    case Rdiff.OP_COPY_N2_N1: {
		int oldOff = (int) readInt(2, data);
		int bs = (int) readInt(1, data);
		Offsets o = new Offsets(oldOff, offset, bs);
		offset += bs;
		return o;
	    }

	    case Rdiff.OP_COPY_N2_N2: {
		int oldOff = (int) readInt(2, data);
		int bs = (int) readInt(2, data);
		Offsets o = new Offsets(oldOff, offset, bs);
		offset += bs;
		return o;
	    }

	    case Rdiff.OP_COPY_N4_N1: {
		int oldOff = (int) readInt(4, data);
		int bs = (int) readInt(1, data);
		Offsets o = new Offsets(oldOff, offset, bs);
		offset += bs;
		return o;
	    }

	    case Rdiff.OP_COPY_N4_N2: {
		int oldOff = (int) readInt(4, data);
		int bs = (int) readInt(2, data);
		Offsets o = new Offsets(oldOff, offset, bs);
		offset += bs;
		return o;
	    }

	    case Rdiff.OP_COPY_N4_N4: {
		int oldOff = (int) readInt(4, data);
		int bs = (int) readInt(4, data);
		Offsets o = new Offsets(oldOff, offset, bs);
		offset += bs;
		return o;
	    }

	    default:
		throw new IOException("Bad delta command: 0x"
			+ Integer.toHexString(command));
	    }
	}
	return null;
    }

    public void stageOut(String attachementName, InputStream stream)
	    throws IOException {
	File output = File.createTempFile("smoarsync", null);
	out = new RandomAccessFile(output, "rw");
	try {
	    in = new RandomAccessFile(new File(origin), "r");
	} catch (IOException ignore) {
	}

	data = new DataInputStream(stream);
	offset = 0;

	int header = readInt(data);
	if (header != Rdiff.DELTA_MAGIC) {
	    throw new IOException("Bad delta header: 0x"
		    + Integer.toHexString(header));
	}

	Delta d;
	while ((d = readDelta()) != null) {
	    applyDelta(d);
	}

	if (in != null)
	    in.close();

	out.close();

	File o = new File(origin);
	o.getParentFile().mkdirs();
	output.renameTo(o);
    }

}