package com.smoa.comp.sdk.types;

import javax.xml.ws.wsaddressing.W3CEndpointReference;

import com.smoa.core.sdk.utils.EndpointReferenceUtils;

public class ActivityEndpointReference {
	protected String address;
	protected String activityUUID;
	protected transient W3CEndpointReference endpointReference;
	
	public String getAddress() {
		return address;
	}

	public String getActivityUUID() {
		return activityUUID;
	}

	public W3CEndpointReference getEndpointReference() {
		return this.getEndpointReference(null);
	}
	
	public W3CEndpointReference getEndpointReference(String serviceAddress){
		address  = address == null ? serviceAddress : address;
		
		if (endpointReference == null) {
			
			StringBuffer sb = new StringBuffer(200);
			
			sb.append("<wsa5:EndpointReference xmlns:wsa5='http://www.w3.org/2005/08/addressing'>");
			sb.append("<wsa5:Address>");
			sb.append(address);
			sb.append("</wsa5:Address>");
			sb.append("<wsa5:ReferenceParameters>");
			sb.append("<ns:ActivityId xmlns:ns='http://schemas.smoa-project.com/comp/2009/01/factory'>");
			sb.append(activityUUID);
			sb.append("</ns:ActivityId>");
			sb.append("</wsa5:ReferenceParameters>");
			sb.append("</wsa5:EndpointReference>");
			
			endpointReference = EndpointReferenceUtils.loadFromString(sb.toString());

		}
			
		return endpointReference;
	}

	
	public ActivityEndpointReference(W3CEndpointReference epr) {
		this.endpointReference = epr;
		
		address = EndpointReferenceUtils.getElementValue(epr, "Address");
		activityUUID = EndpointReferenceUtils.getElementValue(epr, "ActivityId");
		
		if (address == null || activityUUID == null)
			throw new NullPointerException("Either Address or ActivityId is missing in given EPR");
		
	}
	
	public ActivityEndpointReference(String address, String activityUUID) {
		this.address = address;
		this.activityUUID = activityUUID;
	}
	
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result
				+ ((activityUUID == null) ? 0 : activityUUID.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		ActivityEndpointReference other = (ActivityEndpointReference) obj;
		if (activityUUID == null) {
			if (other.activityUUID != null)
				return false;
		} else if (!activityUUID.equals(other.activityUUID))
			return false;
		return true;
	}

	public	ActivityEndpointReference(String reservationId) {
		this(null, reservationId);
	}
	
	@Override
	public String toString() {
		return "ActivityEndpointReference[address=" + address + ",activityUUID=" + activityUUID + "]";
	}

}
