package com.smoa.comp.sdk.types;

import javax.xml.ws.wsaddressing.W3CEndpointReference;

import com.smoa.comp.stubs.bes.factory.ActivityStatusType;
import com.smoa.comp.stubs.factory.ActivityEndStatusType;




public class SMOAActivityStatus {
	

	protected static final String BES_ACTIVITY_STATE_PENDING           = "Pending";
	protected static final String BES_ACTIVITY_STATE_RUNNING           = "Running";
	protected static final String BES_ACTIVITY_STATE_CANCELLED         = "Cancelled";
	protected static final String BES_ACTIVITY_STATE_FAILED            = "Failed";
	protected static final String BES_ACTIVITY_STATE_FINISHED          = "Finished";

	protected static final String BES_ACTIVITY_SMOA_SUBSTATE_STAGE_IN  = "Stage-in";
	protected static final String BES_ACTIVITY_SMOA_SUBSTATE_HELD      = "Held";
	protected static final String BES_ACTIVITY_SMOA_SUBSTATE_QUEUED    = "Queued";
	protected static final String BES_ACTIVITY_SMOA_SUBSTATE_EXECUTING = "Executing";
	protected static final String BES_ACTIVITY_SMOA_SUBSTATE_SUSPENDED = "Suspended";
	protected static final String BES_ACTIVITY_SMOA_SUBSTATE_STAGE_OUT = "Stage-out";
	
	public static final String SMOA_ACTIVITY_STATE_CANCELLED         = BES_ACTIVITY_STATE_CANCELLED;
	public static final String SMOA_ACTIVITY_STATE_FAILED            = BES_ACTIVITY_STATE_FAILED;
	public static final String SMOA_ACTIVITY_STATE_FINISHED          = BES_ACTIVITY_STATE_FINISHED;
	public static final String SMOA_ACTIVITY_STATE_STAGE_IN          = BES_ACTIVITY_SMOA_SUBSTATE_STAGE_IN;
	public static final String SMOA_ACTIVITY_STATE_HELD              = BES_ACTIVITY_SMOA_SUBSTATE_HELD;
	public static final String SMOA_ACTIVITY_STATE_QUEUED            = BES_ACTIVITY_SMOA_SUBSTATE_QUEUED;
	public static final String SMOA_ACTIVITY_STATE_EXECUTING         = BES_ACTIVITY_SMOA_SUBSTATE_EXECUTING;
	public static final String SMOA_ACTIVITY_STATE_SUSPENDED         = BES_ACTIVITY_SMOA_SUBSTATE_SUSPENDED;
	public static final String SMOA_ACTIVITY_STATE_STAGE_OUT         = BES_ACTIVITY_SMOA_SUBSTATE_STAGE_OUT;
	
	protected String stateName;
	protected String subStateName;
	protected ActivityEndStatusType endStatus;
	protected W3CEndpointReference activityIdentifier;
	protected ActivityStatusType raw;
	
	public SMOAActivityStatus(W3CEndpointReference activityIdentifier, ActivityStatusType status) {
		super();
		this.activityIdentifier = activityIdentifier;
		this.endStatus = status.getActivityEndStatus();
		this.stateName = status.getState().value();
		this.raw = status;
		
		if (status.getStageIn() != null) 
			subStateName = SMOAActivityStatus.BES_ACTIVITY_SMOA_SUBSTATE_STAGE_IN;
		else if (status.getStageOut() != null) 
			subStateName = SMOAActivityStatus.BES_ACTIVITY_SMOA_SUBSTATE_STAGE_OUT;
		else if (status.getHeld() != null) 
			subStateName = SMOAActivityStatus.BES_ACTIVITY_SMOA_SUBSTATE_HELD;
		else if (status.getQueued() != null) 
			subStateName = SMOAActivityStatus.BES_ACTIVITY_SMOA_SUBSTATE_QUEUED;
		else if (status.getExecuting() != null) 
			subStateName = SMOAActivityStatus.BES_ACTIVITY_SMOA_SUBSTATE_EXECUTING;
		else if (status.getSuspended() != null) 
			subStateName = SMOAActivityStatus.BES_ACTIVITY_SMOA_SUBSTATE_SUSPENDED;
		else 
			subStateName = null;
	}
	
	public SMOAActivityStatus(W3CEndpointReference activityIdentifier, ActivityEndStatusType endStatus, String stateName,
			String subStateName) {
		super();
		this.activityIdentifier = activityIdentifier;
		this.endStatus = endStatus;
		this.stateName = stateName;
		this.subStateName = subStateName;
	}
	
	public String getBESStateName(){
		return stateName;
	}
	
	public String getBESSubStateName() {
		return subStateName;
	}

	public ActivityEndStatusType getEndStatus() {
		return endStatus;
	}
	
	public String getState() {
		if (subStateName != null) {
			return subStateName;
		} else {
			return stateName;
		}
	}

	public W3CEndpointReference getActivityIdentifier() {
		return activityIdentifier;
	}
	
	public String toString() {
		return "Status[state=" + getState() + (endStatus != null ? ",exitcode=" + endStatus.getExitStatus() : "" ) + (getFaultString()!=null ? getFaultString() : "" ) + "]";
	}
	
	public boolean isFinalState() {
		return (BES_ACTIVITY_STATE_FINISHED.equals(stateName) || BES_ACTIVITY_STATE_FAILED.equals(stateName) || BES_ACTIVITY_STATE_CANCELLED.equals(stateName));
	}
	
	public ActivityStatusType getRAW(){
		return raw;
	}
	
	public String getFaultString() {
		return (raw.getFault() != null ? raw.getFault().getFaultstring() : null);
	}

}
