package com.smoa.comp.tests;

import com.smoa.comp.sdk.ardl.ARDL;
import com.smoa.comp.sdk.exceptions.NotAuthorizedException;
import com.smoa.comp.sdk.exceptions.UnknownReservationIdentifierException;
import com.smoa.comp.stubs.ares.ARESReservationStatusType;
import com.smoa.comp.stubs.ares.ARESReservedHostType;

/**
 * Unit test for simple App.
 */
public class ARBestEffortMultiuserTest extends SMOACompConfig
{
	
	@Override
	protected void setUp() throws Exception {
		super.setUp();
		
	}
	
	public void testReservationTwoCpu() {
		
		try {
		ARDL ardl = new ARDL(70); // create resr for 70 s; 
		ardl.setTotalSlotsCount(1);
		ardl.setAdditionalSlotsCount(2);
		ardl.setUserName(username1);
		ardl.setName("RangSlots");
		resEpr1 = ares.createReservation(ardl);
		System.err.println("Reservation created " + username1 + " = "  + resEpr1);
		
		printReservationStatus(ares.getReservationStatus(resEpr1));

		ARESReservationStatusType resStat =  ares.getReservationStatus(resEpr1);
		int total = 0;
		for (ARESReservedHostType rh : resStat.getReservedHost()) {
			total+=rh.getReservedSlotsCount().intValue();
		}

		System.err.println("total number of cpus reserved = " + total);
		
		assertTrue(total + " >= 1", total >= 1);
		assertTrue(total + " <= 3", total <= 3);
		
		ares.terminateReservation(resEpr1);
		resEpr1 = null;
		
		} catch (Exception ex) {
			ex.printStackTrace();
			
			if (resEpr1 != null) {
				try {
					ares.terminateReservation(resEpr1);
				} catch (NotAuthorizedException e) {
					e.printStackTrace();
				} catch (UnknownReservationIdentifierException e) {
					e.printStackTrace();
				}
			}
			fail();
		} 
			
	}
	
	public void testExclusiveReservation() {
		
		try {
			ARDL ardl = new ARDL(70); // create resr for 70 s; 
			ardl.setTotalSlotsCount(1);
			ardl.setUserName(username1);
			ardl.setName("Exclusive Execution");
			ardl.setExclusiveExecution(true);
			resEpr1 = ares.createReservation(ardl);
			System.err.println("Reservation created " + username1 + " = "  + resEpr1);
			
			printReservationStatus(ares.getReservationStatus(resEpr1));
	
			ARESReservationStatusType resStat =  ares.getReservationStatus(resEpr1);
			
			assertEquals(1, resStat.getReservedHost().size());
			assertEquals(resStat.getReservedHost().get(0).getTotalSlotsCount(), resStat.getReservedHost().get(0).getReservedSlotsCount());
			
			ares.terminateReservation(resEpr1);
			resEpr1 = null;
			
		} catch (Exception ex) {
			ex.printStackTrace();
			if (resEpr1!= null) {
				try {
					ares.terminateReservation(resEpr1);
				} catch (NotAuthorizedException e) {
					e.printStackTrace();
				} catch (UnknownReservationIdentifierException e) {
					e.printStackTrace();
				}
			}
			fail();
		} 
			
	}
}
