package com.smoa.comp.tests;



import java.util.Date;
import java.util.List;

import com.smoa.comp.sdk.ardl.ARDL;
import com.smoa.comp.sdk.jsdl.JSDL;
import com.smoa.comp.sdk.types.ActivityEndpointReference;
import com.smoa.comp.sdk.types.SMOAActivityStatus;
import com.smoa.comp.sdk.types.ReservationEndpointReference;


/**
 * Unit test for simple App.
 */
public class AROneCpuMultiuserTest 
    extends SMOACompConfig
{
	

	@Override
	protected void setUp() throws Exception {
		super.setUp();
		
	}
	
	
	public void testReservationTwoCpu() throws Exception {
		
		ARDL ardl = new ARDL(60); // create resr for 60 s; 

		ardl.setUserName(username1);
		ardl.setName("2");
		ardl.setTotalSlotsCount(1);
		
		//ardl.getCandiateHosts().add("qcg1.lri.fr");
		//ardl.setAdditionalCPUCount(1);
		
		ReservationEndpointReference rEpr1 = ares.createReservation(ardl);
		System.err.println("Reservation created" + username1 + " = "  + rEpr1);
		
		printReservationStatus(ares.getReservationStatus(rEpr1));

		ardl.setUserName(username2);
		//ardl.getCandiateHosts().add("qcg1.lri.fr");
		//ardl.getCandiateHosts().add("qcg4.lri.fr");

		ReservationEndpointReference rEpr2 = ares.createReservation(ardl);
		System.err.println("Reservation created" + username2 + " = "  + rEpr2);
		
		printReservationStatus(ares.getReservationStatus(rEpr2));


		List<ReservationEndpointReference> activeReservations = ares.getActiveReservations();
		
		System.err.println("Active reservations: " + activeReservations.size());
		for (ReservationEndpointReference rEpr : activeReservations) {
			System.err.println(rEpr);
			System.err.println("ResDocument = " + ares.getReservationDocument(rEpr));
			System.err.println("ResLocalId  = " + ares.getReservationDocument(rEpr).getLocalReservationId());
		}
		
		
		JSDL jsdl = new JSDL("sleep");
		jsdl.setExecutable("/bin/sleep");
		jsdl.getArguments().add("10");
		
		jsdl.setUserName(username1);
		jsdl.setReservationIdentifier(rEpr1);
		ActivityEndpointReference jobEpr1 = factory.createActivity(jsdl);
		System.err.println(username1 + ": Job submitted = " + new Date());
		System.err.println(username1 + ": jobEpr = " + jobEpr1);
		
		jsdl.setUserName(username2);
		jsdl.setReservationIdentifier(rEpr2);
		ActivityEndpointReference jobEpr2 = factory.createActivity(jsdl);
		System.err.println(username2 + ": Job submitted = " + new Date());
		System.err.println(username2 + ": jobEpr = " + jobEpr2);
		
		printReservationStatus(ares.getReservationStatus(rEpr1));

		SMOAActivityStatus stat = null;
		while (!(stat = factory.getActivityStatus(jobEpr1)).getState().equals(SMOAActivityStatus.SMOA_ACTIVITY_STATE_EXECUTING)) {
			Thread.sleep(5000);
			System.err.println(stat);
			
		}
		
		System.err.println(username1 + ": Job started = " + new Date());
		
		System.err.println(ares.getReservationDocument(rEpr1));
		System.err.println(ares.getReservationDocument(rEpr2));
		
		printReservationStatus(ares.getReservationStatus(rEpr1));
		printReservationStatus(ares.getReservationStatus(rEpr2));
		
		while (!(stat = factory.getActivityStatus(jobEpr2)).getState().equals(SMOAActivityStatus.SMOA_ACTIVITY_STATE_EXECUTING)) {
			Thread.sleep(5000);
			System.err.println(stat);
		}

		printReservationStatus(ares.getReservationStatus(rEpr1));
		printReservationStatus(ares.getReservationStatus(rEpr2));

		
		System.err.println(username2 + ": Job started = " + new Date());
		
		while (!(stat = factory.getActivityStatus(jobEpr1)).isFinalState()) {
			printReservationStatus(ares.getReservationStatus(rEpr1));
			printReservationStatus(ares.getReservationStatus(rEpr2));
			Thread.sleep(5000);
			printReservationStatus(ares.getReservationStatus(rEpr1));
			printReservationStatus(ares.getReservationStatus(rEpr2));
		}

		assertEquals("Finished", stat.getState());

		System.err.println(username1 + ": Job finished with stat " +  stat + " = " + new Date());
		
		while (!(stat = factory.getActivityStatus(jobEpr1)).isFinalState()) {
			Thread.sleep(5000);
		}
		assertEquals("Finished", stat.getState());
		
		System.err.println(username1 + ": Job finished with stat " +  stat + " = " + new Date());
		
		printReservationStatus(ares.getReservationStatus(rEpr1));
		printReservationStatus(ares.getReservationStatus(rEpr2));

	}
	
}
