package com.smoa.comp.tests;



import java.util.Date;
import java.util.List;

import javax.xml.ws.wsaddressing.W3CEndpointReference;

import com.smoa.comp.sdk.ardl.ARDL;
import com.smoa.comp.sdk.jsdl.JSDL;
import com.smoa.comp.sdk.types.ActivityEndpointReference;
import com.smoa.comp.sdk.types.ReservationEndpointReference;
import com.smoa.comp.sdk.types.SMOAActivityStatus;

/**
 * Unit test for simple App.
 */
public class ARResInfo 
    extends SMOACompConfig
{
	@Override
	protected void setUp() throws Exception {
		super.setUp();
	}
	
	public void testReservationTwoCpu() throws Exception {
		long currTime = ((new Date().getTime()) / (60 * 1000)) * 60 * 1000;
		Date startTime = new Date(currTime + 60 * 1000);
		Date endTime = new Date(currTime + 2 * 60 * 1000);
		
		ARDL ardl = new ARDL(startTime, endTime); // create resr for 120 s; 
		ardl.setTotalSlotsCount(2);
		ardl.setUserName(username1);
		ardl.setName("resName");
		ardl.setDescription("resDesc");
		ardl.getCandiateHosts().add(localHostname);
		
		System.err.println("Reservation for host: " + localHostname);
		
		resEpr1 = ares.createReservation(ardl);
		System.err.println("Reservation created "+ resEpr1);
		
		printReservationStatus(ares.getReservationStatus(resEpr1));
		assertEquals(1, ares.getReservationStatus(resEpr1).getReservedHost().size());
		assertTrue(ares.getReservationStatus(resEpr1).getReservedHost().get(0).getHostName().startsWith(localHostname));
		assertEquals(2, ares.getReservationStatus(resEpr1).getReservedHost().get(0).getReservedSlotsCount().intValue());
		assertEquals(0, ares.getReservationStatus(resEpr1).getReservedHost().get(0).getUsedSlotsCount().intValue());

		ARDL ardl2 = ares.getReservationDocument(resEpr1).getArdl();
		
		assertEquals(startTime, ardl2.getStartTime());
		assertEquals(endTime, ardl2.getEndTime());
		
		System.err.println("Reservation Local Id = " + ares.getReservationDocument(resEpr1).getLocalReservationId());
		assertNotNull(ares.getReservationDocument(resEpr1).getLocalReservationId());

		List<ReservationEndpointReference> activeReservations = ares.getActiveReservations();
		
		assertEquals(1, activeReservations.size());
		assertEquals(resEpr1, activeReservations.get(0));
		
		JSDL jsdl = new JSDL("sleep");
		jsdl.setExecutable("/bin/sleep");
		jsdl.getArguments().add("30");
		jsdl.setUserName(username1);
		jsdl.setReservationIdentifier(resEpr1);

		jobEpr1 = factory.createActivity(jsdl);
		
		waitForState(jobEpr1, SMOAActivityStatus.SMOA_ACTIVITY_STATE_EXECUTING);
		assertEquals(1, ares.getReservationStatus(resEpr1).getReservedHost().get(0).getUsedSlotsCount().intValue());
		List<W3CEndpointReference> aList  = ares.getReservationStatus(resEpr1).getAssociatedActivities().getActivityIdentifier();
		assertEquals(1,aList.size());
		assertEquals(jobEpr1, new ActivityEndpointReference(aList.get(0)));

		ares.terminateReservation(resEpr1);
	}
}
