package com.smoa.comp.tests;

import static org.junit.Assert.*;

import java.io.File;
import java.util.ArrayList;

import org.junit.Before;
import org.junit.Test;

import com.smoa.comp.sdk.SMOAStaging;
import com.smoa.comp.sdk.jsdl.JSDL;
import com.smoa.comp.sdk.jsdl.JSDLDataStaging;
import com.smoa.comp.sdk.types.FileInfo;
import com.smoa.core.sdk.security.AuthenticationModule;
import com.smoa.core.sdk.security.anonymous.AnonymousAuthentication;

public class ListDirectoryStagingTest {

    protected SMOAStaging staging;

    public static final String baseDir = "/tmp";       

    @Before
    public void setUp() throws Exception {
	String hostname = "localhost";
	int port = 19000;
	boolean useWSA = true;
	AuthenticationModule am = new AnonymousAuthentication();

	staging = new SMOAStaging(hostname, port, am, useWSA, null);	
    }

    @Test
    public void listFiles() throws Throwable {
	JSDL jsdl = staging.listDirectory(baseDir, null);		
	File dir = new File(baseDir);
	
	ArrayList<String> local = new ArrayList<String>();	
	
	for(String f : dir.list())
	    local.add(f);
	
	for(JSDLDataStaging ds : jsdl.getDataStaging())
	    local.remove(ds.getFileName());
	
	assertTrue(local.isEmpty());
    }
    
    @Test
    public void statFile() throws Throwable {
	ArrayList<String> files = new ArrayList<String>();
	files.add("/etc/passwd");
	FileInfo[] file = staging.statFile(files, null);
	assertEquals(1, file.length);
	File f=new File("/etc/passwd");	
	assertEquals(f.length(), file[0].getSize());
	assertEquals("root", file[0].getOwner());
    }
    
    @Test
    public void delete() throws Throwable {
	File f=File.createTempFile("staging", "delete");
	assertTrue(f.exists());
	staging.delete(f.getAbsolutePath(), null);
	assertFalse(f.exists());
    }
}
