package com.smoa.comp.tests;



import junit.framework.TestCase;

import com.smoa.comp.sdk.SMOAFactory;
import com.smoa.comp.sdk.jsdl.JSDL;
import com.smoa.comp.sdk.types.ActivityEndpointReference;
import com.smoa.comp.sdk.types.SMOAActivityStatus;
import com.smoa.core.sdk.security.anonymous.AnonymousAuthentication;

/**
 * Unit test for simple Job submission.
 */
public class SMOACompSample 
    extends TestCase
{
	public static final String SMOA_COMPUTING_HOST = "localhost";
	public static final int SMOA_COMPUTING_PORT = 19001;
	
	public void testSubmit() throws Exception {
		SMOAFactory besFactory =  new SMOAFactory(SMOA_COMPUTING_HOST, SMOA_COMPUTING_PORT, new AnonymousAuthentication(), false);
		
		/* create an JSDL document */
		JSDL dateJSDL = new JSDL("SimpleJob");
		
		dateJSDL.setExecutable("/bin/date");
		dateJSDL.getArguments().add("-R");
		dateJSDL.getEnvironment().put("TZ", "Pacific/Honolulu");
		
		/* submit a job */
		ActivityEndpointReference jobEpr = besFactory.createActivity(dateJSDL);
		
		/* wait for termination */
		while (besFactory.getActivityStatus(jobEpr).isFinalState()) { 
			Thread.sleep(2000);
		}
		
		/* get final status */
		SMOAActivityStatus status =  besFactory.getActivityStatus(jobEpr);
		
		if (status.getBESStateName().equals(SMOAActivityStatus.SMOA_ACTIVITY_STATE_FINISHED)) {
			System.out.println("Job " + status.getActivityIdentifier() + " finished with exit status: " + status.getEndStatus().getExitStatus() );
		} else if (status.getBESStateName().equals(SMOAActivityStatus.SMOA_ACTIVITY_STATE_FAILED)){
			System.err.println("Job " + status.getActivityIdentifier() + " failed");
		} else if (status.getBESStateName().equals(SMOAActivityStatus.SMOA_ACTIVITY_STATE_CANCELLED)){
			System.err.println("Job " + status.getActivityIdentifier() + " was cancelled");
		}
		
	}
	
}
