#!/bin/sh

STATUS_OK=0
STATUS_WARNING=1
STATUS_CRITICAL=2
STATUS_UNKNOWN=3

#defaults
SMOA_COMP_HOST="localhost"
SMOA_COMP_PORT="19002"
USER_PROXY="/tmp/x509up_u"`id -u`
SMOA_TEST_JSDL="smoa-test-job.xml"
TIMEOUT="100"

usage() {
	echo usage: `basename $0`' -H hostname -x proxy -p port -t timeout [-v 0-3 -j test-jsdl.xml]'
}

while getopts ":H:t:p:x:j:v:h" OPTION
do
    case $OPTION in
        H)
            SMOA_COMP_HOST=$OPTARG
            ;;
        t)
            TIMEOUT=$OPTARG
            ;;
        x)
            USER_PROXY=$OPTARG
            ;;
        p)
            SMOA_COMP_PORT=$OPTARG
            ;;
	j)
	    SMOA_TEST_JSDL=$OPTARG
            ;;
	v)
	    if [ $OPTARG != "0" ]
	    then
	    	SMOA_DEBUG=$OPTARG
	    fi
	    ;;
        ?)
            echo ERROR: Illegal option: "$OPTARG"
	    usage;
	    exit $STATUS_CRITICAL
	    ;;  
        :)
            echo ERROR: Option: "$OPTARG" requires an argument
	    usage;
	    exit $STATUS_CRITICAL
	    ;;
	h)
	    usage;
            exit $STATUS_UNKNOWN
            ;;
    esac
done

SMOA_COMP_SERVICE_URL="httpg://${SMOA_COMP_HOST}:${SMOA_COMP_PORT}/"

SMOA_PLUGIN_HOME=$(cd `dirname $0`  && pwd)
SMOA_COMP_CLIENT="$SMOA_PLUGIN_HOME/SMOAComputingSDK-2.2.0/smoa-comp-client.sh"
export SMOA_COMP_SERVICE_URL
export X509_USER_PROXY="$USER_PROXY"

if [ ! -x $SMOA_COMP_CLIENT ]
then
    echo SMOA COMP STATUS: SMOA Comp Client not found!
    exit $STATUS_UNKNOWN
fi

if [ ! -f $SMOA_TEST_JSDL ]
then
    echo SMOA COMP STATUS: Test job file \'$SMOA_TEST_JSDL\' not found!
    exit $STATUS_UNKNOWN
fi

if [ -n "$SMOA_DEBUG" ]
then
	echo SMOA_COMP_SERVICE_URL=$SMOA_COMP_SERVICE_URL
	echo SMOA_TEST_JSDL=$SMOA_TEST_JSDL
fi


SMOA_OUTPUT=`exec 2>&1 && $SMOA_COMP_CLIENT -c -J $SMOA_TEST_JSDL`
ACTIVITY_ID=`echo "$SMOA_OUTPUT" | grep UUID | cut -d : -f 4`

if [ -n "$SMOA_DEBUG" ]
then
    echo ACTIVITY_ID=$ACTIVITY_ID
fi

if [  "$SMOA_DEBUG" == "3" ]
then
    echo "$SMOA_OUTPUT"
fi


if [ -z $ACTIVITY_ID ]
then
    SMOA_EXCEPTION=`echo "$SMOA_OUTPUT" | grep Exception`
    echo SMOA STATUS: Failed to submit a job: $SMOA_EXCEPTION
    exit $STATUS_CRITICAL
fi


START_TIME=`date +%s`
ELAPSED_TIME=0

while [ $ELAPSED_TIME -lt $TIMEOUT ]
do
    SMOA_OUTPUT=`exec 2>&1 && $SMOA_COMP_CLIENT -s -a "urn:uuid:$ACTIVITY_ID"`
    STATUS=`echo "$SMOA_OUTPUT" | grep Status | cut -d ' ' -f 4`
    ELAPSED_TIME=$((`date +%s` - $START_TIME))
    
    if [ -n "$SMOA_DEBUG" ]
    then
	echo ITERATION=$i STATUS=$STATUS
    fi

    if [  "$SMOA_DEBUG" == "3" ]
    then
    	echo "$SMOA_OUTPUT"
    fi



    if [ "x"`echo $STATUS | grep Finished `  != "x" ]
    then
	EXIT_CODE=`echo $STATUS | cut -d "=" -f 3 | cut -d "]" -f 1 `

	if [ $EXIT_CODE -eq 0 ]
	then
		echo "SMOA STATUS: Job finished successfully after $ELAPSED_TIME seconds."
		exit $STATUS_OK
	else
                echo "SMOA STATUS: Job finished with exitcode: $EXIT_CODE"
		exit $STATUS_WARNING
	fi
    fi

    if [  "x"`echo $STATUS | grep Failed`  != "x" ]
    then
        echo "SMOA STATUS: Test job ($SMOA_TEST_JSDL) failed" 
	exit $STATUS_CRITICAL
    fi

    if  [  "x"`echo $STATUS | grep Cancelled`  != "x" ]
    then
	echo "SMOA STATUS: Test job Cancelled??"
	exit $STATUS_CRITICAL
    fi
    
    sleep 5

    ELAPSED_TIME=$((`date +%s` - $START_TIME))
done

echo SMOA STATUS: Job has not been finished \(STATUS=$STATUS\) after $ELAPSED_TIME seconds!
exit $STATUS_WARNING
